import * as THREE from 'three';
export default {
  init: function init(props) {
    var _props$background = props.background,
        background = _props$background === void 0 ? null : _props$background,
        _props$fog = props.fog,
        fog = _props$fog === void 0 ? null : _props$fog,
        _props$overrideMateri = props.overrideMaterial,
        overrideMaterial = _props$overrideMateri === void 0 ? null : _props$overrideMateri,
        _props$autoUpdate = props.autoUpdate,
        autoUpdate = _props$autoUpdate === void 0 ? true : _props$autoUpdate,
        _props$onFrame = props.onFrame,
        onFrame = _props$onFrame === void 0 ? function () {} : _props$onFrame;
    var Scene = new THREE.Scene();

    if (fog !== null) {
      if (Object.hasOwnProperty.call(fog, 'density')) {
        Scene.fog = new THREE.FogExp2(fog.color, fog.density);
      } else {
        Scene.fog = new THREE.Fog(fog.color, fog.near, fog.far);
      }
    }

    Scene.background = background;
    Scene.overrideMaterial = overrideMaterial;
    Scene.autoUpdate = autoUpdate;
    Scene.onFrame = onFrame;
    Scene.updateProperty = this.updateProperty;
    return Scene;
  },
  updateProperty: function updateProperty(key, value) {
    if (key === 'fog') {
      if (Object.hasOwnProperty.call(value, 'density')) {
        this.fog = new THREE.FogExp2(value.color, value.density);
      } else {
        this.fog = new THREE.Fog(value.color, value.near, value.far);
      }

      return;
    }

    if (Object.hasOwnProperty.call(this, key)) {
      if (this[key] && this[key].set) {
        return this[key].set.apply(this[key], Object.values(value));
      }

      this[key] = value;
      return;
    }
  }
};