"use strict";

exports.__esModule = true;
exports.default = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _components = _interopRequireDefault(require("./components"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var CHILDREN = 'children';
var EVENT_PREFIX_REGEXP = /^on[A-Z]/;
var ID = 'uuid';
var nodeMaps = {};
var renderer = null;
var scene = null;
var camera = null;

function _createElement(type, props) {
  return _components.default[type].init(props);
}

var Driver = {
  getElementById: function getElementById(id) {
    return nodeMaps[id];
  },
  createBody: function createBody() {
    if (document.getElementsByTagName('canvas').length !== 0) {
      return document.getElementsByName('canvas').shift();
    }

    renderer = new THREE.WebGLRenderer();
    renderer.setPixelRatio(window.devicePixelRatio);
    renderer.setSize(window.innerWidth, window.innerHeight);
    document.body.appendChild(renderer.domElement);
    return renderer;
  },
  createEmpty: function createEmpty() {
    return _createElement({
      type: 'object-3d'
    });
  },
  createElement: function createElement(type, props) {
    var node = _createElement(type, props);

    this.setNativeProps(node, props);
    return node;
  },
  appendChild: function appendChild(node, parent) {
    if (node.type === 'Scene') {
      node.renderer = renderer;
      scene = node;
      return;
    }

    if (node.isCamera) {
      camera = node;
      return;
    }

    if (parent.type === 'Scene' && node.isObject3D) {
      parent.add(node);
    } else {
      if (parent.isObject3D) {
        if (node.isGeometry || node.isBufferGeometry) parent.geometry = node;
        if (node.isMaterial) parent.material = node;
      }
    }

    return renderer.render(scene, camera);
  },
  removeChild: function removeChild(node, parent) {
    parent = parent || node.parent;
    var id = node[ID];

    if (id != null) {
      nodeMaps[id] = null;
    }

    return parent.remove(node);
  },
  replaceChild: function replaceChild(newChild, oldChild, parent) {
    parent = parent || oldChild.parent;
    this.removeChild(oldChild, parent);
    this.appendChild(newChild, parent);
  },
  addEventListener: function addEventListener(node, eventName, eventHandler) {
    if (node.type === 'Scene') {
      return document.body.addEventListener(eventName, eventHandler);
    }

    return node;
  },
  removeEventListener: function removeEventListener(node, eventName, eventHandler) {
    if (node.type === 'Scene') {
      return document.body.removeEventListener(eventName, eventHandler);
    }

    return node;
  },
  removeAttribute: function removeAttribute(node, propKey, propValue) {// Noop
  },
  setAttribute: function setAttribute(node, propKey, propValue) {
    if (propKey == ID) {
      nodeMaps[propValue] = node;
    }

    node.updateProperty(propKey, propValue);
    return node;
  },
  afterRender: function afterRender() {
    render();
  },
  setNativeProps: function setNativeProps(node, props) {
    for (var prop in props) {
      var value = props[prop];

      if (prop === CHILDREN) {
        continue;
      }

      if (value != null) {
        if (EVENT_PREFIX_REGEXP.test(prop)) {
          var eventName = prop.slice(2).toLowerCase();
          this.addEventListener(node, eventName, value);
        } else {
          this.setAttribute(node, prop, value);
        }
      }
    }
  }
};

var render = function render() {
  window.requestAnimationFrame(render);
  renderer.render(scene, camera);
  scene.onFrame();
};

var _default = Driver;
exports.default = _default;